@@ -229,7 +229,7 @@ class MainMenu(_filledMenu, EventMixin):
                         _("Copy the input strings of the cells to clipboard"),
                         wx.ID_COPY]],
                 [item, [self.CopyResultMsg,
-                        _("Copy &Results") + "\tShift+Ctrl+c",
+                        _("Copy &results") + "\tShift+Ctrl+c",
                         _("Copy the result strings of the cells to the "
                           "clipboard")]],
                 [item, [self.PasteMsg, _("&Paste") + "\tCtrl+v",
@@ -273,6 +273,10 @@ class MainMenu(_filledMenu, EventMixin):
                 [item, [self.ShowResizeGridDialogMsg, _("Resize grid"),
                         _("Resize grid")]]]],
             [wx.Menu, _("&View"), [
+                [item, [self.FullScreenMsg, _("Fullscreen") + "\tF11",
+                        _("Shows grid in fullscreen mode. "
+                          "Press <F11> to leave.")]],
+                ["Separator"],
                 [wx.Menu, _("Toolbars"), [
                     [item, [self.MainToolbarToggleMsg, _("Main toolbar"),
                             _("Shows and hides the main toolbar.")],
@@ -329,15 +333,14 @@ class MainMenu(_filledMenu, EventMixin):
                 [item, [self.FontStrikethroughMsg, _("Strikethrough"),
                         _("Toggles strikethrough.")]],
                 ["Separator"],
-                [item, [self.FrozenMsg, _("Frozen"),
-                        _("Toggles frozen state of cell. ") +
-                        _("Frozen cells are updated only "
-                          "when F5 is pressed.")]],
-                [item, [self.LockMsg, _("Lock"),
-                        _("Lock cell. Locked cells cannot be changed.")]],
-                [item, [self.MergeMsg, _("Merge cells"),
-                        _("Merges / unmerges selected cells. ")]],
-
+                [item, [self.TextColorDialogMsg, _("Text color..."),
+                        _("Launch color dialog to specify text color.")]],
+                [item, [self.BgColorDialogMsg, _("Background color..."),
+                        _("Launch color dialog to specify background "
+                          "color.")]],
+                ["Separator"],
+                [item, [self.MarkupMsg, _("Markup"),
+                        _("Show cell result as Pango markup")]],
                 ["Separator"],
                 [wx.Menu, _("Justification"), [
                     [item, [self.JustificationMsg, _("Left"), _("Left"),
@@ -350,15 +353,26 @@ class MainMenu(_filledMenu, EventMixin):
                 [wx.Menu, _("Alignment"), [
                     [item, [self.AlignmentMsg, alignment, alignment]]
                     for alignment in [_("Top"), _("Center"), _("Bottom")]]],
+                [wx.Menu, _("Rotation"), [
+                    [item, [self.Rotation0Msg,
+                            _("0°"), _("Set text rotation to 0°")]],
+                    [item, [self.Rotation90Msg,
+                            _("90°"), _("Set text rotation to 90°")]],
+                    [item, [self.Rotation180Msg,
+                            _("180°"), _("Set text rotation to 180°")]],
+                    [item, [self.Rotation270Msg,
+                            _("270°"), _("Set text rotation to 270°")]],
+                ]],
                 ["Separator"],
-                [item, [self.TextColorDialogMsg, _("Text color..."),
-                        _("Launch color dialog to specify text color.")]],
-                [item, [self.BgColorDialogMsg, _("Background color..."),
-                        _("Launch color dialog to specify background "
-                          "color.")]],
-                ["Separator"],
-                [item, [self.RotationDialogMsg, _("Rotation..."),
-                        _("Set text rotation.")]]]],
+                [item, [self.FrozenMsg, _("Frozen"),
+                        _("Toggles frozen state of cell. ") +
+                        _("Frozen cells are updated only "
+                          "when F5 is pressed.")]],
+                [item, [self.LockMsg, _("Lock"),
+                        _("Lock cell. Locked cells cannot be changed.")]],
+                [item, [self.MergeMsg, _("Merge cells"),
+                        _("Merges / unmerges selected cells. ")]],
+                ]],
             [wx.Menu, _("&Macro"), [
                 [item, [self.MacroListMsg, _("&Macro list") + "\tCtrl+m",
                         _("Choose, fill in, manage, and create macros")]],
